/* sorgente: DinMatIONome.c */

/* illustra le modalit di allocazione dinamica, di acquisizione e di
** restituzione, e di rilascio della memoria, per una matrice di interi */

/* inclusione del file di intestazione della libreria standard
** che contiene definizioni di macro, costanti e dichiarazioni
** di funzioni e tipi funzionali alle varie operazioni di I/O */
#include <stdio.h>

/* inclusione del file di intestazione della libreria standard
** che contiene definizioni di macro, costanti e dichiarazioni
** di funzioni di interesse generale */
#include <stdlib.h>

/*
** definizione della funzione per il recupero della memoria allocata
** per una matrice di interi
*/
void FreeMatInt(int **Mat, unsigned int righe)
  { 
  /* definisce una variabile per l'indice di riga */
  unsigned int riga;

  /* rilascia la memoria allocata per ognuna delle righe della matrice */
  for(riga=0; riga< righe; riga++)
    free(Mat[riga]);
    
  /* rilascia la memoria allocata per il vettore di accesso alle righe
  ** della matrice */
  free(Mat);
  }
  
/*
** definizione della funzione per l'allocazione a run-time della memoria
** per una matrice di interi
*/
int **AllMatInt(unsigned int righe, unsigned int colonne)
  {
  /* definizione di una variabile per l'indirizzo di accesso alla matrice e di una
  ** per lindice di riga */
  int** Mat;
  unsigned int riga;

  /* inizializzazione della variabile di accesso alla matrice con l'indirizzo di 
  ** un vettore di indirizzi di variabili intere di dimensione pari al 
  ** numero delle righe */
  Mat = (int **) malloc(righe*sizeof(int *));
  
  /* verifica dell'esito della inizializzazione, se negativo termina con NULL */
  if (Mat == NULL)
    return(NULL);
  
  /* inizializzazione di ogni elemento del vettore con l'indirizzo di un
  ** vettore di variabili intere di dimensione pari al numero delle colonne */
  for(riga=0; riga< righe; riga++)
    {
    Mat[riga] = (int *)malloc(colonne*sizeof(int));
    
    /* verifica dell'esito della inizializzazione, se negativo rilascia la
    ** memoria allocata fino a quel momento e termina con NULL */
    if (Mat[riga] == NULL)
      {
      FreeMatInt(Mat, riga-1);
      
      return(NULL);
      }
    };
  
    /* restituzione dell'indirizzo di accesso alla matrice */
    return(Mat);
    }

/*
** definizione della funzione per l'acquisizione del contenuto di una matrice di interi
*/
void AcqMatInt(int **Mat, unsigned int dim_righe, unsigned int dim_col)
  {
  /* definizione delle variabili per l'indice di riga e quello di colonna */
  unsigned int riga, col;
  
  /* scansione delle righe della matrice */
  for (riga = 0; riga < dim_righe; riga = riga+1)
    /* scansione delle righe della matrice */
    for (col = 0; col < dim_col; col = col+1) 
      {
      /* acquisizione dell'elemento corrente della matrice */
      printf("\nMat[%d][%d]: ", riga, col);
      scanf("%d", &(Mat[riga][col]));
      };
  }
  
/*
** definizione della funzione per la restituzione del contenuto di una matrice di interi
*/ 

void ResMatInt(int **Mat, unsigned int dim_righe, unsigned int dim_col)
  {
  /* definizione delle variabili per l'indice di riga e quello di colonna */
  size_t riga, col;
  
  /* scansione delle righe della matrice */
  for (riga = 0; riga < dim_righe; riga = riga+1)
    /* scansione delle righe della matrice */
    for (col = 0; col < dim_col; col = col+1)
      {
      /* va a capo se necessario */
      if (col == 0)
        printf("\n");
      
      /* visualizzazione dell'elemento corrente */  
      printf("%4d", Mat[riga][col]);
      }
  printf("\n");
  }

/* testing delle funzioni */

int main ()
  {
  /* definizione delle variabili per l'indirizzo di accesso alla matrice
  ** e per il numero di righe e di colonne */
  int** Matrice;
  unsigned int nro_righe, nro_col;
  
  /* acquisizione del numero di righe e di colonne della matrice */
  printf("Nro righe? ");
  scanf("%d", &nro_righe);
  
  printf("Nro colonne? ");
  scanf("%d", &nro_col);
  
  /* alloca memoria per la matrice e assegna l'indirizzo di accesso
  ** alla variabile preposta */
  Matrice = AllMatInt(nro_righe, nro_col);

  /* verifica dell'esito dell'allocazione, se negativo termina con 0*/
  if (Matrice == NULL)
    {
    printf("\nAllocazione di memoria fallita");
    return(0);
    };
  
  /* acquisisce, e successiva restituzione, del contenuto della matrice */
  AcqMatInt(Matrice, nro_righe, nro_col);
  ResMatInt(Matrice, nro_righe, nro_col);
  
  /* recupero della memoria allocata per la matrice */
  FreeMatInt(Matrice, nro_righe);
  
  /* termina con 1 */
  return(1);
  } 
